package swingtables.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.font.TextAttribute;
import java.sql.Date;
import java.util.Map;
import java.util.regex.PatternSyntaxException;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.RowFilter;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableRowSorter;
import swingtables.logic.DataSource;
/*
 * Készítsünk grafikus felületű alkalmazást, amely Wikipedia oldalakról gyűjt
 * össze információkat, és ezeket egy adatrácsban jeleníti meg.
 * A szükséges információk:
 * - az oldal azonosítója - egész szám
 * - az oldal címe - szöveg
 * - az oldal webcíme - URL
 * - az oldal hossza - egész szám
 * - az oldal utolsó módosításának ideje - dátum
 * 
 * A program DataSource osztálya a Wiki oldalakat véletlenszerűen tudja
 * összeszedni, ezeket gyűjti a memóriában. Csak egy táblamodellt és egy
 * adatrácsot kell hozzá készítenünk.
 * A tábla:
 * - rendelkezzen a megfelelő fejlécekkel
 * - a cellákban az objektumokat a szokásos formátumban jelenítse meg
 * - ne legyen szerkeszthető
 * - legyen rendezhető bármelyik oszlop szerint
 * - legyen szűrhető
 * -- a megadott szótöredéket tartalmazó oldalcímekre
 * -- a megadott hossznál nagyobb hosszúságú oldalakra
 * -- a megadott időpontnál később szerkesztett oldalakra
 */

public class SwingTablesFrame extends JFrame {

    public static void main(String[] args) {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                new SwingTablesFrame().setVisible(true);
            }
        });
    }
    private final WikipediaPageInfoTableModel wpiTableModel;
    private final JTable wpiTable;
    private final TableRowSorter<WikipediaPageInfoTableModel> sorterAndFilter;
    private JTextField filterTextField;

    public SwingTablesFrame() throws HeadlessException {
        setTitle("Wikipedia page infos");
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        setLocation(40, 60);
        setSize(900, 500);
        setLayout(new BorderLayout());
        JToolBar jToolBar = new JToolBar();
        getContentPane().add(jToolBar, BorderLayout.NORTH);
        jToolBar.add(startLoaderAction);
        jToolBar.add(stopLoaderAction);
        stopLoaderAction.setEnabled(false);
        wpiTableModel = new WikipediaPageInfoTableModel();
        wpiTable = new JTable(wpiTableModel);
        getContentPane().add(new JScrollPane(wpiTable), BorderLayout.CENTER);
        wpiTable.getColumn("URL").setCellRenderer(new TableCellRenderer() {
            @Override
            public Component getTableCellRendererComponent(JTable table, final Object value, final boolean isSelected, final boolean hasFocus, int row, int column) {
                JPanel jpanel = new JPanel() {
                    @Override
                    public void paint(Graphics g) {
                        if (isSelected) {
                            g.setColor(Color.DARK_GRAY);
                        } else {
                            g.setColor(Color.WHITE);
                        }
                        g.drawRect(0, 0, getWidth(), getHeight());
                        g.setColor(Color.BLUE);
                        Map<TextAttribute, Object> attributes = (Map<TextAttribute, Object>) g.getFont().getAttributes();
                        attributes.put(TextAttribute.WEIGHT, 1.0);
                        attributes.put(TextAttribute.FOREGROUND, Color.BLUE);
                        attributes.put(TextAttribute.UNDERLINE, 1);
                        if (isSelected) {
                            attributes.put(TextAttribute.BACKGROUND, Color.LIGHT_GRAY);
                        }
                        g.setFont(g.getFont().deriveFont(attributes));
                        g.drawString(value.toString(), 2, 12);
                    }
                };
                return jpanel;
            }
        });
        sorterAndFilter = new TableRowSorter<>(wpiTableModel);
        wpiTable.setRowSorter(sorterAndFilter);
        jToolBar.add(new JToolBar.Separator());
        filterTextField = new JTextField();
        jToolBar.add(filterTitleAction);
        jToolBar.add(filterLengthAction);
        jToolBar.add(filterDateAction);
        jToolBar.add(clearFilterAction);
        jToolBar.add(filterTextField);
    }
    private final Action startLoaderAction = new AbstractAction("Start loader") {
        @Override
        public void actionPerformed(ActionEvent e) {
            DataSource.getInstance().startLoading();
            switchActionsEnabled();
        }
    };
    private final Action stopLoaderAction = new AbstractAction("Stop loading") {
        @Override
        public void actionPerformed(ActionEvent e) {
            DataSource.getInstance().stopLoading();
            switchActionsEnabled();
        }
    };

    private void switchActionsEnabled() {
        startLoaderAction.setEnabled(!startLoaderAction.isEnabled());
        stopLoaderAction.setEnabled(!stopLoaderAction.isEnabled());
    }

    @Override
    public void dispose() {
        stopLoaderAction.actionPerformed(null);
        super.dispose();
    }
    private final Action filterTitleAction = new AbstractAction("Filter titles") {
        @Override
        public void actionPerformed(ActionEvent e) {
            RowFilter<WikipediaPageInfoTableModel, Object> rf;
            try {
                rf = RowFilter.regexFilter(filterTextField.getText(), 0);
            } catch (PatternSyntaxException ex) {
                return;
            }
            setRowFilter(rf);
        }
    };
    private final Action filterLengthAction = new AbstractAction("Filter lengths") {
        @Override
        public void actionPerformed(ActionEvent e) {
            RowFilter<WikipediaPageInfoTableModel, Object> rf;
            try {
                rf = RowFilter.numberFilter(RowFilter.ComparisonType.AFTER, Long.parseLong(filterTextField.getText()), 3);
            } catch (PatternSyntaxException | NumberFormatException ex) {
                return;
            }
            setRowFilter(rf);
        }
    };
    private final Action filterDateAction = new AbstractAction("Filter last modifications") {
        @Override
        public void actionPerformed(ActionEvent e) {
            RowFilter<WikipediaPageInfoTableModel, Object> rf;
            try {
                rf = RowFilter.dateFilter(RowFilter.ComparisonType.AFTER, Date.valueOf(filterTextField.getText()), 4);
            } catch (IllegalArgumentException ex) {
                return;
            }
            setRowFilter(rf);
        }
    };
    private final Action clearFilterAction = new AbstractAction("Clear filter") {
        @Override
        public void actionPerformed(ActionEvent e) {
            setRowFilter(null);
        }
    };

    private void setRowFilter(RowFilter<WikipediaPageInfoTableModel, Object> rf) {
        try {
            sorterAndFilter.setRowFilter(rf);
        } catch (NullPointerException npe) {
        }
    }
}
